@@ -77,7 +77,7 @@ rubrica_file_is_rubrica_doc(xmlDocPtr doc, gchar *filename)
       g_strcasecmp(doc->children->name, "Rubrica") != 0)
     {      
       rubrica_dialog_error_loading_file(filename, 
-					RE_XML_DOC_NO_RUBRICA_HEADER);   
+					RE_NO_RUBRICA_HEADER);   
       xmlFreeDoc(doc);
 
       return FALSE;
@@ -967,7 +967,7 @@ rubrica_file_parse_doc(Rubrica *rubrica, xmlDocPtr doc, RubricaErrorType *err)
 	}
     }
   else
-    *err = RE_XML_DOC_NO_RUBRICA_HEADER;
+    *err = RE_NO_RUBRICA_HEADER;
 
   *err = RE_NO_ERROR;
 
