@@ -29,9 +29,8 @@
 #include <time.h>
 
 #include "vcard.h"
-#include "rubrica_vcard_marshal.h"
 
-#define BUFLEN    128
+#define BUFLEN    1024
 #define ARRAY_LEN 8
 
 #define TOKEN_DELIM ':'
@@ -47,6 +46,31 @@ RubricaVCardValueType oldvalue[ARRAY_LEN];
 gchar* tmpbuf = NULL;
 gboolean multiline_data = FALSE;
 
+struct _RubricaVCardData {
+  gchar* fn;              /* FN */
+  gchar* n;               /* N => family name */
+  gchar* gn;              /* N => given name */
+  gchar* an;              /* N => additional names */
+  gchar* hp;              /* N => honorific prefix */
+  gchar* hs;              /* N => honorific suffix */
+  gchar* nick;            /* nickname */
+  gchar* photo;           /* photo (uri or inline binary) - todo */
+  gchar* bday;            /* birthday */
+  gchar* label;           /* label (postal address) */
+  gchar* title;           /* job title or functional position */
+  gchar* role;            /* role, occupation or business category */
+  gchar* org;             /* organization */
+  gchar* note;            /* */
+  gchar* class;           /* */
+  gchar* key;             /* public key */
+
+  GList* email;           /* */ 
+  GList* url;             /* */
+  GList* adr;             /* address (RubricaVCardAddress list) */
+  GList* tel;             /* telephone (RubricaVCardTelephone list) */
+  GList* cat;             /* categories (groups list) */    
+};
+
 
 static RubricaVCardType tokens[] = { 
   {"N",   N},             {"FN",  FN},        {"TZ",  TZ},
@@ -218,6 +242,8 @@ void   rubrica_vcard_set_tel      (RubricaVCard* vcard, gpointer tel);
 
 
 struct _RubricaVCardPrivate {
+  RubricaVCardData *data;
+
   gboolean dispose_has_run;
 };
 
@@ -261,13 +287,14 @@ rubrica_vcard_init(RubricaVCard* vcard)
 
   vcard->fp  = NULL;
   
-  vcard->data = g_malloc(sizeof(RubricaVCardData));
-  if (!vcard->data)
-    g_error("\nRubricaVCard: Out of memory!");
-
-  rubrica_vcard_data_init(vcard->data);
-
   vcard->private = g_new(RubricaVCardPrivate, 1);
+
+  vcard->private->data = g_malloc(sizeof(RubricaVCardData));
+  if (!vcard->private->data)
+    g_error("\nRubricaVCard: Out of memory!");
+  
+  rubrica_vcard_data_init(vcard->private->data);
+
   vcard->private->dispose_has_run = FALSE;
 }
 
@@ -289,7 +316,7 @@ rubrica_vcard_class_init(RubricaVCardClass* klass)
 		 G_STRUCT_OFFSET(RubricaVCardClass, vcard_file_opened),
 		 NULL,
 		 NULL,
-		 rubrica_vcard_marshal_VOID__INT,
+		 g_cclosure_marshal_VOID__INT,
 		 G_TYPE_NONE,         /* return type */
 		 1,                   /* params      */
 		 G_TYPE_INT);         /* params type */
@@ -301,7 +328,7 @@ rubrica_vcard_class_init(RubricaVCardClass* klass)
 		 G_STRUCT_OFFSET(RubricaVCardClass, vcard_decoded_item),
 		 NULL,
 		 NULL,
-		 rubrica_vcard_marshal_VOID__VOID,
+		 g_cclosure_marshal_VOID__VOID,
 		 G_TYPE_NONE,
 		 0,
 		 G_TYPE_NONE);
@@ -313,7 +340,7 @@ rubrica_vcard_class_init(RubricaVCardClass* klass)
 		 G_STRUCT_OFFSET(RubricaVCardClass, vcard_decoded),
 		 NULL,
 		 NULL,
-		 rubrica_vcard_marshal_VOID__VOID,
+		 g_cclosure_marshal_VOID__VOID,
 		 G_TYPE_NONE,
 		 0,
 		 G_TYPE_NONE);
@@ -337,9 +364,9 @@ rubrica_vcard_finalize (RubricaVCard* vcard)
 {
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   
-  rubrica_vcard_data_clean(vcard->data);
+  rubrica_vcard_data_clean(vcard->private->data);
 
-  g_free(vcard->data);
+  g_free(vcard->private->data);
   g_free(vcard->private);
 }
 
@@ -797,7 +824,7 @@ rubrica_vcard_build_card(RubricaVCard *vcard,
   switch(token)
     {
     case BEGIN:
-      rubrica_vcard_data_clean(vcard->data);
+      rubrica_vcard_data_clean(vcard->private->data);
       break;
       
     case END:
@@ -918,9 +945,10 @@ rubrica_vcard_build_card(RubricaVCard *vcard,
       
     case TEL:
       /* telefono + tipo */   
-      while (*varray)
+      if (*varray)
 	{
 	  RubricaVCardValueType value;
+	  gboolean unknown;
 	  gchar* tmp;
 	  
 	  tt = (RubricaVCardTelephone*)g_malloc(sizeof(RubricaVCardTelephone));
@@ -932,47 +960,54 @@ rubrica_vcard_build_card(RubricaVCard *vcard,
 	    }
 
 	  rubrica_vcard_set_tel_num(tt, data);
-	  value = (RubricaVCardValueType) *varray;
       
-	  switch (value)
-	    {	  
-	    case HOME:
-	      tmp = "home";
-	      break;
-	    case WORK:
-	      tmp = "work";
-	      break;
-	    case FAX:
-	      tmp = "fax";
-	      break;
-	    case CELL:
-	      tmp = "cellphone";
-	      break;
-	      
-	    case BAD_VALUE:
-	      tmp = "unknown";
-	      break;
-	      
-	      /* other telephone type */
-	    case PREF:
-	    case VOICE:
-	    case MSG:
-	    case PAGER:
-	    case BBS:
-	    case MODEM:
-	    case CAR:
-	    case ISDN:
-	    case VIDEO:
-	    default:
-	      tmp = "other";
-	      break;
-	    }
+	  do
+	    {
+	      unknown = FALSE;
+	      value = (RubricaVCardValueType) *varray;
 
+	      switch (value)
+		{	  
+		case HOME:
+		  tmp = "home";
+		  break;
+		case WORK:
+		  tmp = "work";
+		  break;
+		case FAX:
+		  tmp = "fax";
+		  break;
+		case CELL:
+		  tmp = "cellphone";
+		  break;
+		  
+		case BAD_VALUE:
+		  tmp = "unknown";
+		  unknown = TRUE;
+		  break;
+		  
+		  /* other telephone type */
+		case PREF:
+		case VOICE:
+		case MSG:
+		case PAGER:
+		case BBS:
+		case MODEM:
+		case CAR:
+		case ISDN:
+		case VIDEO:
+		default:
+		  tmp = "other";
+		  unknown = TRUE;				  
+		  break;
+		}
+	      
+	      varray++;
+	    } while (*varray && unknown == TRUE);
+	  
 	  rubrica_vcard_set_tel_type(tt, tmp);
-	  varray++;
+	  rubrica_vcard_set_tel(vcard, tt);
 	}
-
-      rubrica_vcard_set_tel(vcard, tt);
       break;
       
     case EMAIL:  
@@ -1141,18 +1176,18 @@ rubrica_vcard_optimize_card(RubricaVCard* vcard)
 {
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
 
-  if (!vcard->data->fn)
+  if (!vcard->private->data->fn)
     {
-      if (vcard->data->n && vcard->data->gn)
-	vcard->data->fn = g_strconcat(vcard->data->n, " ", 
-				      vcard->data->gn, NULL);
-      else if (vcard->data->n)
-	vcard->data->fn = g_strdup(vcard->data->n);
-      else if (vcard->data->gn)
-	vcard->data->fn = g_strdup(vcard->data->gn);
-      else if (vcard->data->org)
-	vcard->data->fn = g_strdup(vcard->data->org);
-      else vcard->data->fn = g_strdup("no name card");
+      if (vcard->private->data->n && vcard->private->data->gn)
+	vcard->private->data->fn = g_strconcat(vcard->private->data->n, " ", 
+					       vcard->private->data->gn, NULL);
+      else if (vcard->private->data->n)
+	vcard->private->data->fn = g_strdup(vcard->private->data->n);
+      else if (vcard->private->data->gn)
+	vcard->private->data->fn = g_strdup(vcard->private->data->gn);
+      else if (vcard->private->data->org)
+	vcard->private->data->fn = g_strdup(vcard->private->data->org);
+      else vcard->private->data->fn = g_strdup("no name card");
     }
 }
 
@@ -1163,7 +1198,7 @@ rubrica_vcard_set_fn(RubricaVCard* vcard, gchar* str)
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   g_return_if_fail(str != NULL);
 
-  vcard->data->fn = g_strdup(str);
+  vcard->private->data->fn = g_strdup(str);
 }
 
 
@@ -1176,9 +1211,9 @@ rubrica_vcard_set_n(RubricaVCard* vcard, gchar* str)
   if (strcmp(str, "") == 0)
     return;
 
-//    vcard->data->n = NULL;
+//    vcard->private->data->n = NULL;
 //  else
-  vcard->data->n = g_strdup(str);
+  vcard->private->data->n = g_strdup(str);
 }
 
 
@@ -1190,9 +1225,9 @@ rubrica_vcard_set_gn(RubricaVCard* vcard, gchar* str)
 
   if (strcmp(str, "") == 0)
     return;
-//    vcard->data->gn = NULL;
+//    vcard->private->data->gn = NULL;
 //  else
-  vcard->data->gn = g_strdup(str);
+  vcard->private->data->gn = g_strdup(str);
 }
 
 
@@ -1204,9 +1239,9 @@ rubrica_vcard_set_an(RubricaVCard* vcard, gchar* str)
 
   if (strcmp(str, "") == 0)
     return;
-//    vcard->data->an = NULL;
+//    vcard->private->data->an = NULL;
 //  else
-  vcard->data->an = g_strdup(str);  
+  vcard->private->data->an = g_strdup(str);  
 }
 
 
@@ -1218,9 +1253,9 @@ rubrica_vcard_set_hp(RubricaVCard* vcard, gchar* str)
   
   if (strcmp(str, "") == 0)
     return;
-//    vcard->data->hp = NULL;
+//    vcard->private->data->hp = NULL;
 //  else
-    vcard->data->hp = g_strdup(str); 
+    vcard->private->data->hp = g_strdup(str); 
 }
 
 
@@ -1231,11 +1266,11 @@ rubrica_vcard_set_hs(RubricaVCard* vcard, gchar* str)
   g_return_if_fail(str != NULL);
 
   if (strcmp(str, "") == 0)
-//    vcard->data->hs = NULL;
+//    vcard->private->data->hs = NULL;
 //  else
     return;
 
-  vcard->data->hs = g_strdup(str);  
+  vcard->private->data->hs = g_strdup(str);  
 }
 
 
@@ -1365,7 +1400,7 @@ rubrica_vcard_set_adr(RubricaVCard* vcard, gpointer adr)
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   g_return_if_fail(adr != NULL);
       
-  vcard->data->adr = g_list_append(vcard->data->adr, adr);
+  vcard->private->data->adr = g_list_append(vcard->private->data->adr, adr);
 }
 
 
@@ -1395,7 +1430,7 @@ rubrica_vcard_set_tel(RubricaVCard* vcard, gpointer tel)
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   g_return_if_fail(tel != NULL);
 
-  vcard->data->tel = g_list_append(vcard->data->tel, tel);
+  vcard->private->data->tel = g_list_append(vcard->private->data->tel, tel);
 }
 
 
@@ -1405,7 +1440,7 @@ rubrica_vcard_set_email(RubricaVCard* vcard, gchar* str)
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   g_return_if_fail(str != NULL);
 
-  vcard->data->email = g_list_append(vcard->data->email, g_strdup(str));
+  vcard->private->data->email = g_list_append(vcard->private->data->email, g_strdup(str));
 }
 
 
@@ -1415,7 +1450,7 @@ rubrica_vcard_set_url(RubricaVCard* vcard, gchar* str)
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   g_return_if_fail(str != NULL);
 
-  vcard->data->url = g_list_append(vcard->data->url, g_strdup(str));  
+  vcard->private->data->url = g_list_append(vcard->private->data->url, g_strdup(str));  
 }
 
 
@@ -1428,7 +1463,7 @@ rubrica_vcard_set_bday(RubricaVCard* vcard, gchar* str)
   g_return_if_fail(str != NULL);
   
   t = rubrica_vcard_decode_date(str, FALSE);
-  vcard->data->bday = g_strdup_printf("%d", t);
+  vcard->private->data->bday = g_strdup_printf("%d", t);
 }
 
 
@@ -1438,7 +1473,7 @@ rubrica_vcard_set_title(RubricaVCard* vcard, gchar* str)
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   g_return_if_fail(str != NULL);
 
-  vcard->data->title = g_strdup(str);  
+  vcard->private->data->title = g_strdup(str);  
 }
 
 
@@ -1448,7 +1483,7 @@ rubrica_vcard_set_org(RubricaVCard* vcard, gchar* str)
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   g_return_if_fail(str != NULL);
 
-  vcard->data->org = g_strdup(str);  
+  vcard->private->data->org = g_strdup(str);  
 }
 
 
@@ -1458,7 +1493,7 @@ rubrica_vcard_set_role(RubricaVCard* vcard, gchar* str)
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   g_return_if_fail(str != NULL);
 
-  vcard->data->role = g_strdup(str);  
+  vcard->private->data->role = g_strdup(str);  
 }
 
 
@@ -1470,19 +1505,19 @@ rubrica_vcard_set_note(RubricaVCard* vcard, gchar* str)
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   g_return_if_fail(str != NULL);
 
-  if (vcard->data->note)
+  if (vcard->private->data->note)
     {
-//      tmp = g_strdup(vcard->data->note);
-//      g_free(vcard->data->note);
-      gchar *tmp = vcard->data->note;
-      vcard->data->note = g_strconcat(tmp, str, NULL);  
+//      tmp = g_strdup(vcard->private->data->note);
+//      g_free(vcard->private->data->note);
+      gchar *tmp = vcard->private->data->note;
+      vcard->private->data->note = g_strconcat(tmp, str, NULL);  
       g_free(tmp);
     }
 //  else tmp = "";
 
-//  vcard->data->note = g_strconcat(tmp, str, NULL);  
+//  vcard->private->data->note = g_strconcat(tmp, str, NULL);  
   else 
-    vcard->data->note = g_strdup(str);
+    vcard->private->data->note = g_strdup(str);
 }
 
 
@@ -1492,7 +1527,7 @@ rubrica_vcard_set_key(RubricaVCard* vcard, gchar* str)
   g_return_if_fail(IS_RUBRICA_VCARD(vcard));
   g_return_if_fail(str != NULL);
 
-  vcard->data->key = g_strdup(str);  
+  vcard->private->data->key = g_strdup(str);  
 }
 
 
@@ -1531,7 +1566,7 @@ void rubrica_vcard_clean_array(RubricaVCardParamType parray[],
 }
 
 
-gint          
+gint 
 rubrica_vcard_open_file (RubricaVCard* vcard, gchar* filename)
 {
   gchar buffer[BUFLEN]; 
@@ -1588,24 +1623,19 @@ rubrica_vcard_open_file (RubricaVCard* vcard, gchar* filename)
 		  g_signal_emit_by_name(vcard, "vcard_decode_card", 
 					vcard, G_TYPE_NONE);
 		}
-	      
 	    }
 
 	  g_string_free(string, TRUE);
 	  memset(buffer, 0, BUFLEN);
 	  rubrica_vcard_clean_array(parray, varray);
 
-/*
-	  memset(parray, 0, ARRAY_LEN);
-	  memset(varray, 0, ARRAY_LEN);
-*/
 	  string = g_string_new(NULL);	  
 	}
     }
   while(!feof(vcard->fp));
-
+  
   g_signal_emit_by_name(vcard, "vcard_decode_addressbook", NULL, G_TYPE_NONE);
-
+  
   fclose(vcard->fp);
   g_string_free(string, TRUE);
   
@@ -1617,9 +1647,9 @@ gchar*
 rubrica_vcard_get_fn (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->fn;
+  return (gchar*) vcard->private->data->fn;
 }
 
 
@@ -1627,9 +1657,9 @@ gchar*
 rubrica_vcard_get_n (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->n;
+  return (gchar*) vcard->private->data->n;
 }
 
 
@@ -1637,9 +1667,9 @@ gchar*
 rubrica_vcard_get_gn (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->gn;
+  return (gchar*) vcard->private->data->gn;
 }
 
 
@@ -1647,9 +1677,9 @@ gchar*
 rubrica_vcard_get_an (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->an;
+  return (gchar*) vcard->private->data->an;
 }
 
 
@@ -1657,9 +1687,9 @@ gchar*
 rubrica_vcard_get_hp (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->hp;
+  return (gchar*) vcard->private->data->hp;
 }
 
 
@@ -1667,9 +1697,9 @@ gchar*
 rubrica_vcard_get_hs (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->hs;
+  return (gchar*) vcard->private->data->hs;
 }
 
 
@@ -1677,9 +1707,9 @@ RubricaVCardAddress*
 rubrica_vcard_get_adr (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), NULL);
-  g_return_val_if_fail(vcard->data != NULL, NULL);
+  g_return_val_if_fail(vcard->private->data != NULL, NULL);
   
-  return (gpointer) vcard->data->adr;
+  return (gpointer) vcard->private->data->adr;
 }
 
 
@@ -1687,9 +1717,9 @@ RubricaVCardTelephone*
 rubrica_vcard_get_tel (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), NULL);
-  g_return_val_if_fail(vcard->data != NULL, NULL);
+  g_return_val_if_fail(vcard->private->data != NULL, NULL);
   
-  return (gpointer) vcard->data->tel;
+  return (gpointer) vcard->private->data->tel;
 }
 
 
@@ -1697,12 +1727,12 @@ gchar*
 rubrica_vcard_get_email (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  if (!vcard->data->email)
+  if (!vcard->private->data->email)
     return NULL;
   
-  return (gchar*) vcard->data->email->data;
+  return (gchar*) vcard->private->data->email->data;
 }
 
 
@@ -1710,12 +1740,12 @@ gchar*
 rubrica_vcard_get_url (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  if (!vcard->data->url)
+  if (!vcard->private->data->url)
     return NULL;
   
-  return (gchar*) vcard->data->url->data;
+  return (gchar*) vcard->private->data->url->data;
 }
 
 
@@ -1723,9 +1753,9 @@ gchar*
 rubrica_vcard_get_bday (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->bday;
+  return (gchar*) vcard->private->data->bday;
 }
 
 
@@ -1733,9 +1763,9 @@ gchar*
 rubrica_vcard_get_title (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->title;
+  return (gchar*) vcard->private->data->title;
 }
 
 
@@ -1743,9 +1773,9 @@ gchar*
 rubrica_vcard_get_org (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->org;
+  return (gchar*) vcard->private->data->org;
 }
 
 
@@ -1753,9 +1783,9 @@ gchar*
 rubrica_vcard_get_role (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->role;
+  return (gchar*) vcard->private->data->role;
 }
 
 
@@ -1763,9 +1793,9 @@ gchar*
 rubrica_vcard_get_note (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->note;
+  return (gchar*) vcard->private->data->note;
 }
 
 
@@ -1773,9 +1803,9 @@ gchar*
 rubrica_vcard_get_key (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  return (gchar*) vcard->data->key;
+  return (gchar*) vcard->private->data->key;
 }
 
 
@@ -1785,15 +1815,15 @@ rubrica_vcard_get_pbox (RubricaVCard* vcard)
   RubricaVCardAddress* adr;
 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
   
-  if (!vcard->data->adr)
+  if (!vcard->private->data->adr)
     return NULL;
 
-  if (!vcard->data->adr->data)
+  if (!vcard->private->data->adr->data)
     return NULL;
 
-  adr = vcard->data->adr->data;
+  adr = vcard->private->data->adr->data;
   
   return (gchar*) adr->pbox;
 }
@@ -1805,15 +1835,15 @@ rubrica_vcard_get_extadd (RubricaVCard* vcard)
   RubricaVCardAddress* adr;
 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
 
-  if (!vcard->data->adr)
+  if (!vcard->private->data->adr)
     return NULL;
 
-  if (!vcard->data->adr->data)
+  if (!vcard->private->data->adr->data)
     return NULL;
 
-  adr = vcard->data->adr->data;
+  adr = vcard->private->data->adr->data;
 
   return (gchar*) adr->extadd;
 }
@@ -1825,15 +1855,15 @@ rubrica_vcard_get_str (RubricaVCard* vcard)
   RubricaVCardAddress* adr;
 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
 
-  if (!vcard->data->adr)
+  if (!vcard->private->data->adr)
     return NULL;
 
-  if (!vcard->data->adr->data)
+  if (!vcard->private->data->adr->data)
     return NULL;
 
-  adr = vcard->data->adr->data;  
+  adr = vcard->private->data->adr->data;  
 
   return (gchar*) adr->str;
 }
@@ -1845,15 +1875,15 @@ rubrica_vcard_get_loc (RubricaVCard* vcard)
   RubricaVCardAddress* adr;
 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
 
-  if (!vcard->data->adr)
+  if (!vcard->private->data->adr)
     return NULL;
 
-  if (!vcard->data->adr->data)
+  if (!vcard->private->data->adr->data)
     return NULL;
 
-  adr = vcard->data->adr->data;
+  adr = vcard->private->data->adr->data;
 
   return (gchar*) adr->loc;
 }
@@ -1865,15 +1895,15 @@ rubrica_vcard_get_reg (RubricaVCard* vcard)
   RubricaVCardAddress* adr;
 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
 
-  if (!vcard->data->adr)
+  if (!vcard->private->data->adr)
     return NULL;
 
-  if (!vcard->data->adr->data)
+  if (!vcard->private->data->adr->data)
     return NULL;
 
-  adr = vcard->data->adr->data;  
+  adr = vcard->private->data->adr->data;  
 
   return (gchar*) adr->reg;
 }
@@ -1885,15 +1915,15 @@ rubrica_vcard_get_code (RubricaVCard* vcard)
   RubricaVCardAddress* adr;
 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
 
-  if (!vcard->data->adr)
+  if (!vcard->private->data->adr)
     return NULL;
 
-  if (!vcard->data->adr->data)
+  if (!vcard->private->data->adr->data)
     return NULL;
 
-  adr = vcard->data->adr->data;
+  adr = vcard->private->data->adr->data;
 
   return (gchar*) adr->code;
 }
@@ -1905,15 +1935,15 @@ rubrica_vcard_get_count (RubricaVCard* vcard)
   RubricaVCardAddress* adr;
 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
 
-  if (!vcard->data->adr)
+  if (!vcard->private->data->adr)
     return NULL;
 
-  if (!vcard->data->adr->data)
+  if (!vcard->private->data->adr->data)
     return NULL;
 
-  adr = vcard->data->adr->data;
+  adr = vcard->private->data->adr->data;
 
   return (gchar*) adr->count;
 }
@@ -1925,15 +1955,15 @@ rubrica_vcard_get_adr_type (RubricaVCard* vcard)
   RubricaVCardAddress* adr;
 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
 
-  if (!vcard->data->adr)
+  if (!vcard->private->data->adr)
     return NULL;
 
-  if (!vcard->data->adr->data)
+  if (!vcard->private->data->adr->data)
     return NULL;
 
-  adr = vcard->data->adr->data;
+  adr = vcard->private->data->adr->data;
 
   return (gchar*) adr->type;
 }
@@ -1945,15 +1975,15 @@ rubrica_vcard_get_tel_num (RubricaVCard* vcard)
   RubricaVCardTelephone* tt;
 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
 
-  if (!vcard->data->tel)
+  if (!vcard->private->data->tel)
     return NULL;
 
-  if (!vcard->data->tel->data)
+  if (!vcard->private->data->tel->data)
     return NULL;
 
-  tt = vcard->data->tel->data;
+  tt = vcard->private->data->tel->data;
 
   return (gchar*) tt->num;
 }
@@ -1965,15 +1995,15 @@ rubrica_vcard_get_tel_type (RubricaVCard* vcard)
   RubricaVCardTelephone* tt;
 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), "");
-  g_return_val_if_fail(vcard->data != NULL, "");
+  g_return_val_if_fail(vcard->private->data != NULL, "");
 
-  if (!vcard->data->tel)
+  if (!vcard->private->data->tel)
     return NULL;
 
-  if (!vcard->data->tel->data)
+  if (!vcard->private->data->tel->data)
     return NULL;
 
-  tt = vcard->data->tel->data;  
+  tt = vcard->private->data->tel->data;  
 
   return (gchar*) tt->type;
 }
@@ -1984,9 +2014,9 @@ rubrica_vcard_next_email (RubricaVCard* vcard)
 { 
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), FALSE);
 
-  vcard->data->email = g_list_next(vcard->data->email);
+  vcard->private->data->email = g_list_next(vcard->private->data->email);
 
-  return (vcard->data->email != NULL);
+  return (vcard->private->data->email != NULL);
 }
 
 gboolean 
@@ -1994,9 +2024,9 @@ rubrica_vcard_next_url (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), FALSE);
 
-  vcard->data->url = g_list_next(vcard->data->url);
+  vcard->private->data->url = g_list_next(vcard->private->data->url);
 
-  return (vcard->data->url != NULL);
+  return (vcard->private->data->url != NULL);
 }
 
 gboolean 
@@ -2004,9 +2034,9 @@ rubrica_vcard_next_adr (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), FALSE);
 
-  vcard->data->adr = g_list_next(vcard->data->adr);
+  vcard->private->data->adr = g_list_next(vcard->private->data->adr);
 
-  return (vcard->data->adr != NULL);
+  return (vcard->private->data->adr != NULL);
 }
 
 gboolean 
@@ -2014,7 +2044,7 @@ rubrica_vcard_next_tel (RubricaVCard* vcard)
 {
   g_return_val_if_fail(IS_RUBRICA_VCARD(vcard), FALSE);
 
-  vcard->data->tel = g_list_next(vcard->data->tel);
+  vcard->private->data->tel = g_list_next(vcard->private->data->tel);
 
-  return (vcard->data->tel != NULL);
+  return (vcard->private->data->tel != NULL);
 }
